; boot.asm
;-----------------------------------------------------------------------------
;               DSP56002EVM initialization code
;
; V1.0 3.Jan.1997
;-----------------------------------------------------------------------------
;****************************************************************************
;* Copyright (C) 1992-1995 by Alef Null. All rights reserved.               *
;* Author(s): Jarkko Vuori, OH2LNS                                          *
;****************************************************************************
;* Here are all DSP56001/2 interrupt vectors and space for                  *
;* interrupt contexts and basic interrupt service routines.                 *
;* Also included are initalization code, self checking and                  *
;* support routines for host communication, codec control,                  *
;* low level AX25 handling and transmit control.                            *
;*                                                                          *
;* KISS protocol handling is based on article                               *
;*      Chepponis, M., Karn, P.:                                            *
;*      "The KISS TNC: A simple Host-to-TNC communications     		    *
;*       protocol",                                            		    *
;*      Proc. of the sixth ARRL computer networking cnf., 1988              *
;*                                                                          *
;* HDLC protocol handling is based on article                               *
;*      Carlson, D., E.:                                                    *
;*      "Bit-Oriented Data Link Control Procedures",                        *
;*      IEEE Trans. on Comm., Vol. 28, No. 4, April 1980                    *
;*                                                                          *
;* CRC calculation/checking is based on article                             *
;*      Morse, G.:                                                          *
;*      "Calculating CRCs by bits and bytes",                               *
;*      BYTE Vol. 11, No. 9, September 1986                                 *
;*                                                                          *
;* Modification(s):                                                         *
;*                                                                          *
;*      08.04.95    added version number printout after reset               *
;*      03.08.95    added DWAIT control logic to the KISS                   *
;*                  transmit control                                        *
;*      02.09.95    corrected and modified KISS parameter                   *
;*                  handling and TXDELAY=0,TXTAIL=0 bugs                    *
;*                  KISS buffer length increased to 2048 bytes              *
;* -------------------------------------------------------------------------*
;* J.B. Forrer, KC7WW for Motorola EVM56K                                   *
;*    Thanks for suggestions and help from P. Jalocha, SP9VRC               *                                                       *
;-------------------------------------------------------------------------- *
; NOTE: r7 is sacred!! - it is used by the SSI - DO NOT TOUCH!              *
;       r3 as well     - it is used by SCI                                  *
;-------------------------------------------------------------------------- *
; Version number (source code date)                                         *
;===========================================================================*
year    equ     1997
month   equ     1
day     equ     3

	page    132

	section bios
	xdef    opensci,putchar,getchar,tstc,endc,rejc,putbit,getbit
	xdef    stimer,putio,caron,caroff

	include 'ioequlc.inc'
	include 'intequlc.inc'
	include 'bios.inc'

;-----------------------------------------------------------------------------
; Define the PLL multiplier
;-----------------------------------------------------------------------------
	if (xtal==32000000)
ClockMult equ   $7              ; set PLL 8x
	endif
	if (xtal==40000000)
ClockMult equ   $9              ; set PLL 10x
	endif
	if (xtal==66000000)     ; NOT DIVISIBLE BY 4 !!! (16.5)
ClockMult equ   $0F             ; set PLL 16x
	endif
	if (xtal==68000000)
ClockMult equ   $10             ; set PLL 17x
	endif
	if (xtal==80000000)
ClockMult equ   $12             ; set PLL 20x
	endif
;-----------------------------------------------------------------------------
; KISS special characters
fend    equ     $c0
fesc    equ     $db
tfend   equ     $dc
tfesc   equ     $dd

; HDLC equations
flagmsk equ     $ff                                 ; left justified flag mask
flag    equ     %01111110                           ; HDLC bit flag
abrtmsk equ     $fe                                 ; left justified abort mask
abort   equ     %11111110                           ; abort sequence
fivemsk equ     $fe                                 ; left justified five bit mask
five    equ     %01111100                           ; left justified five bit sequence
poly    equ     $8408                               ; HDLC CRC polynomial (x^16 + x^12 + x^5 + 1)
crcchk  equ     $f0b8                               ; special CRC checkword

; flags
rempty  equ     0                                   ; SCI flags
rfull   equ     1
xempty  equ     2
xfull   equ     3
timer   equ     4                                   ; timer flag
scmode  equ     5                                   ; serial communication mode
xkissf  equ     6                                   ; put KISS frame begin
ztstflg equ     7                                   ; HDLC xmitter status flags
zinsflg equ     8
hunt    equ     9                                   ; HDLC receiver status flags
firstb  equ     10
scndb   equ     11
pwrup   equ     12                                  ; set if power-up reset
carrier equ     13                                  ; carrier on/off
givedat equ     14                                  ; data output gate

;-----------------------------------------------------------------------------
;                       56K interrupt vectors
;-----------------------------------------------------------------------------

; Reset vector
	org     p:i_reset
	jmp     boot

; Stack error interrupt
	org     p:i_stack
	jmp     shdown

; Trace interrupt
	org     p:i_trace
	jmp     shdown

; SWI
	org     p:i_swi
	jmp     shdown

; IRQA
	org     p:i_irqa
	jmp     shdown

; IRQB
	org     p:i_irqb
	jmp     shdown


; SSI transmitter interrupt
;(because syncronous mode, we can use the same interrupt for both reading and writing)
	org     p:i_ssitd
	movep               y:(r7)+,x:m_tx
	movep               x:m_rx,x:(r7)

; SSI transmitter interrupt with errors
	org     p:i_ssitde
	movep               x:m_sr,x:m_tx           ; clear TUE

; SCI receive interrupt
	org     p:i_scird
	jsr     sci_rec

; SCI receive interrupt with errors
	org     p:i_scirde
	jsr     sci_rec

; SCI transmitter interrupt
	org     p:i_scitd
	jsr     sci_xmt

; SCI timer interrupt
	org     p:i_scitm
	jsr     sci_tim


; System error shutdown
shdown  nop
	stop

; Illegal instruction interrupt
	org     p:$003e
	jmp     >shdown


;*****************************************************************************
;* The following code will be placed to top p-memory bank                    *
;*****************************************************************************

	org     p:topmem

; KISS parameter table (here because of fixed address)
kisspar dc      @cvi(50*baud/100.0)                 ; txdelay
	dc      63                                  ; P
	dc      @cvi(10*baud/100.0)                 ; SlotTim
	dc      @cvi(1*baud/100.0)                  ; TXtail
	dc      0                                   ; FullDup
kissext equ     *-kisspar
	dup     16-(*-kisspar)
	dc      0                                   ; extra KISS parameters
	endm
kisses  equ     *-kisspar

;-----------------------------------------------------------------------------
; processor initialization
;-----------------------------------------------------------------------------
;****************************
;*   Startup code for EVM   *
;****************************

boot    movep   #$261000+ClockMult,x:PLL  ; set PLL

	movep   #$0000,x:m_bcr          ; zero wait states for ext. memory
	ori     #3,mr                   ; disable interrupts
	movec   #0,sp
	move    #0,omr                  ; single chip mode

; initialize SCI
	movep               #$2b02,x:m_scr          ; 8,n,1
	movep               #(xtal+2*16*baud)/(2*2*16*baud)-1,x:m_sccr    ; round baud

; initialize port B
	movep               #$0000,x:m_pbc          ; port B as general purpose port
	movep               #$60ff,x:m_pbddr        ; PB0-PB7,PB13 and PB14 as outputs
	movep               #$0000,x:m_pbd

; initialize port C
	movep               #$0003,x:m_pcc          ; We are using TXD,RXD

						    ; pc2 (D/C~), pc3 (DCD),
	movep               #$1C,x:m_pcddr          ; pc4 (C Reset) as outputs

	movep               #0,x:m_pcd             ; init PC4,PC3,PC2=0

; initialize data structures
	move                #outbuf,a1              ; SCI queue handling
	move                a1,x:<xhead
	move                a1,x:<xtail
	move                #inbuf,a1
	move                a1,x:<rhead
	move                a1,x:<rtail
	move                #mybuflen-1,m3

	move                #@cvi(30/hbeat*baud),n3 ; SCI timer system (hearbeat rate 72 pulses/min)
	clr     a
	move                a,y:<timchg
	move                a,y:<timcnt

	move                #<(1<<rempty)|(1<<xempty),a1
	move                a1,y:<flags             ; buffer empty at first

	move                #wakeup,a1
	move                a1,x:<seqptr


; start interrupts
	movep               #$b000,x:m_ipr          ; SSI=IPL2, SCI=IPL1
	andi    #$fc,mr                             ; unmask interrupts

; flash red LED once to show all is OK
	cmdled  set
	move                #>@cvi(1.0*baud),x0     ; set timer
	jsr     stimer
_wchr   wait
	jset    #timer,y:<flags,_wchr   ; wait for timer
	cmdled  clr

; EVM56K transfer to label "user_code" here
	jmp     user_code
;----------------------------------------------------------------------------
;****************************
;*    SCI timer interrupt   *
;****************************
sci_tim enter   scidata                             ; only a1,a0,x1,x0 are saved

; increment timer
	move                #-1,m3                  ; yes, set next time
	move                y:<timcnt,r3

; check if destination time reached
	move                y:<timval,a1
	move                r3,x0
	eor     x0,a        (r3)+
	jne     _scit1
	bclr    #timer,y:<flags                     ; yes, clear flag bit

; check if watch-dog interval reached
_scit1  move                y:<timchg,a1
	eor     x0,a        r3,y:<timcnt
	jne     scite
	copled  chg
	move                (r3)+n3
	nop
	move                r3,y:<timchg

scite   jsr     pertick                             ; xmit control module timer
	move                #mybuflen-1,m3
	leave   scidata


;****************************
;*    SCI xmit interrupt    *
;****************************
sci_xmt enter   scidata                             ; only a1,a0,x1,x0 are saved

	move                x:<xtail,r3
	move                x:<xhead,x0
	movep               p:(r3)+,x:m_stxl
	move                r3,x:<xtail

	bclr    #xfull,y:<flags                     ; don't bother to check buffer state if it is full
	jcs     scixe

; check if buffer empty
	move    r3,a1
	eor     x0,a
	jne     scixe

; yes, shut down xmitter
	bset    #xempty,y:<flags
	bclr    #m_tie,x:m_scr

scixe   leave   scidata


;****************************
;*   SCI receive interrupt  *
;****************************
wakeup  dc      $12,$b9,$b0,$a1
sci_rec enter   scidata                             ; only a1,a0,x1,x0 are saved

	movep               x:m_ssr,x1              ; clear SCI errors

; first check for special wake-up sequence
	move                x:<seqptr,r3
	movep               x:m_srxl,x1             ; read byte to x1
	move                p:(r3)+,a1
	eor     x1,a        #>wakeup+4,x0
	jeq     scir1
	move                #wakeup,r3
	jmp     scir2
scir1   move                r3,a1
	eor     x0,a
	jne     scir2

; wake-up sequence detected --- NOT FOR BIOS usage!
	stop                    ; JBF

; no wake-up sequence detected, continue searching
scir2   move                r3,x:<seqptr

; then check that there are room left in the buffer
	jset    #rfull,y:<flags,scire

; yes, determine in which mode we are (normal, KISS mode)
	jset    #scmode,y:<flags,scir3

; * normal mode, put data to buffer
	move                x:<rhead,r3
	bclr    #rempty,y:<flags
	move                x1,p:(r3)+
	move                r3,x:<rhead

; check buffer full condition
	move                r3,a1
	move                x:<rtail,x0
	eor     x0,a
	jne     scire
	bset    #rfull,y:<flags

	jmp     scire

; * KISS protocol mode, read received character
scir3   move                x:<getkst,r3
	move                #fend,a1
	jmp     (r3)                                ; determine what to do for it

; --- State 0, waiting for FEND
str0    eor     x1,a        #>str1,x0
	jne     scire                               ; we didn't see FEND, keep looking
	move    x0,x:<getkst                        ; FEND found, change state
	jmp     scire

; --- State 1, we have seen FEND, look for the command byte
str1    eor     x1,a        #>str2,x0
	jeq     scire                               ; just another FEND, keep looking for cmd
; analyze command byte
	move                #0,a1
	eor     x1,a        x1,x:<kisscmd
	jeq     str1b                               ; cmd 0, data will follow
; store cmd and change state
str1a   move                #str4,a1
	move                a1,x:<getkst
	jmp     scire
; start a new frame
str1b   move                x0,x:<getkst
	move                x:<rhead,a1             ; get current rhead
	move                a1,x:<rnhead
	jmp     scire

; --- State 2, data to follow
str2    eor     x1,a                                ; check if end of frame
	jeq     strend
	move                #fesc,a1
	eor     x1,a                                ; check if escape
	jne     store
; escape character found
	move                #str3,a1                ; enter FESC found state
	move                a1,x:<getkst
	jmp     scire
; end of frame, store negative value
strend  move                #-1,x1
	jmp     store

; --- State 3, saw FESC, expecting TFESC or TFEND
str3    move                #tfesc,a1               ; check if TFESC
	eor     x1,a
	jeq     str3esc
	move                #tfend,a1
	eor     x1,a                                ; check if TFEND
	jeq     str3end
	move                #str2,a1
	move                a1,x:<getkst            ; something wrong has happened,
	jmp     scire                               ; go back to the data receiving mode
; we have seen TFESC after an FESC, write an FESC
str3esc move                #str2,a1
	move                a1,x:<getkst
	move                #>fesc,x1
	jmp     store
; we have seen TFEND after an FESC, write an FEND
str3end move                #str2,a1
	move                a1,x:<getkst
	move                #>fend,x1

; store the character to the queue
store   move                x:<rnhead,r3
	move                x:<rtail,x0
	move                x1,p:(r3)+
	move    r3,a1                               ; check buffer full
	eor     x0,a        r3,x:<rnhead
	jeq     scr_st0                             ; queue full, discard current frame
; check if end of frame
	move                #-1,a1
	eor     x1,a        x:<rnhead,x1
	jne     scire

	move                x1,x:<rhead             ; yes, reset real buffer write pointer
	bclr    #rempty,y:<flags
	jsr     frmrec                              ; inform xmit control module
	jmp                 scr_st0

; --- State 4, get command data
str4    move                a2,y:<tmp               ; backup a2
	move                x:<kisscmd,a            ; check if a local parameter
	move                #>kisses,x0
	cmp     x0,a        #>P,x0
	jgt     str4c

	cmp     x0,a        #>kissext,x0            ; local parameter, check if a time parameter
	jeq     str4a
	cmp     x0,a        #>kisspar-1,x0
	jle     str4b

str4a   move                #>kisspar-1,x0
	add     x0,a                                ; no, store it without any conversion
	move                a,r3
	nop
	move                x1,p:(r3)
	jmp     str4c

str4b   add     x0,a        #>baud/100,x0           ; yes, time scale parameter
	mpy     x0,x1,a     a,r3
	asr     a                                   ; interger multiply correction
	move                a0,p:(r3)               ; product in low order word

str4c   move                y:<kisssub,r3           ; give also the parameter to the user application
	move                x:<kisscmd,a
	jsr     (r3)                                ; a1 - cmd, x1 - data
	move                y:<tmp,a2

; go back to FEND hunt state
scr_st0 move                #str0,a1
	move                a1,x:<getkst

scire   leave   scidata

;****************************
;*     Open Serial line     *
;****************************
; flushes all buffers and set the desired communication speed
;   a - kiss command routine address (zero if not is kiss mode)
;   b - xmit on/off routine address
opensci move                a,y:<kisssub
	bclr    #scmode,y:<flags
	tst     a           #str0,a1
	jeq     opensce

; KISS mode, initialize handlers
	move                b1,y:<xmitsub           ; store given addresses
	move                a1,x:<getkst
	move                #(1<<xoff),b1
	clr     b           b1,x:<pstate            ; and initialize xmit control module
	move                b1,y:<pertim

	bset    #scmode,y:<flags
	bclr    #xkissf,y:<flags

	movi    #xstD,y:<xstate                     ; initialize coder
	movi    #flag,y:<xdata
	movi    #$0,y:<x5bit
	movi    #0,y:<xbit
	bclr    #ztstflg,y:<flags
	bclr    #zinsflg,y:<flags

	movi    #0,x:<rdata                         ; initialize decoder
	movi    #$0,x:<rflag
	bset    #hunt,y:<flags
	bset    #firstb,y:<flags
	bset    #scndb,y:<flags

opensce rts


;****************************
;*  Put character to queue  *
;****************************
; byte in x0
; returns  Z if buffer full
;         NZ otherwise
putchar ori     #$02,mr                             ; disable interrupts
	nop
	nop
	jset    #scmode,y:<flags,putkiss

; * normal mode
	move                x:<xtail,x1
	move                x:<xhead,a1
	move                a1,r3

; xmitter was running, check if there are free space left
	bclr    #xempty,y:<flags                    ; if buffer empty, sure there are free space left
	jcs     putc1
	eor     x1,a                                ; if read ptr <> write ptr there are also free space left
	jne     putc2

; buffer full state reached (ignore given data)
	bset    #xfull,y:<flags
	jmp     putce

; there was free space left, write character to the buffer
putc1   bset    #m_tie,x:m_scr                      ; start xmitter interrupts
putc2   move                x0,p:(r3)+
	move                r3,x:<xhead
	jmp     putce


; * KISS mode, check if start of new KISS frame
putkiss jset    #xkissf,y:<flags,putk1

; yes, send KISS preamble
	bset    #xkissf,y:<flags
	move                x:<xhead,r3
	move                r3,x:<xnhead

	move                #fend,a1
	move                a1,p:(r3)+
	move                #0,a1
	move                a1,p:(r3)+

	move                r3,x:<xnhead

; no, send pure data only
putk1   move                x:<xnhead,r3

	move                #fesc,a1                ; check if FESC
	eor     x0,a        #tfesc,b1
	jeq     putkspe

	move                #fend,a1                ; check if FEND
	eor     x0,a        #tfend,b1
	jne     putke1

; special character, enter escaped special character
putkspe move                #>fesc,x0
	move                x0,p:(r3)+
	move    b1,x0

putke1  move                x0,p:(r3)+
	move                r3,x:<xnhead

putce   andi    #$fc,mr
	rts


;****************************
;* End current KISS frame   *
;****************************
endc    ori     #$02,mr                             ; disable interrupts
	nop
	nop
	nop
	bclr    #xkissf,y:<flags

	move                x:<xnhead,r3            ; write last fend
	move                #>fend,x0
	move                x0,p:(r3)+
	move                r3,x:<xhead

	jclr    #xempty,y:<flags,ekisse             ; check for idling xmitter
	bset    #m_tie,x:m_scr

ekisse  andi    #$fc,mr
	rts


;****************************
;* Reject current KISS frame*
;****************************
rejc    bclr    #xkissf,y:<flags
	rts


;****************************
;* Get character from queue *
;****************************
; byte in x0
; returns  C if no data available
;         NC if data available
getchar
	ori     #$02,mr                             ; disable interrupts
	nop
	nop
	nop
	nop
	move                x:<rtail,r3

; check if there are data available
	btst    #rempty,y:<flags
	jcs     getce

; yes, take it from the queue
	bclr    #rfull,y:<flags
	move                p:(r3)+,x0
	move                r3,x:<rtail

; check if buffer gets empty
	move                x:<rhead,a
	move                r3,x1
	cmp     x1,a
	andi    #$fe,ccr                            ; NC
	jne     getce

; yes, set empty flag
	bset    #rempty,y:<flags

getce   andi    #$fc,mr
	rts


;****************************
;* Test if chrs available   *
;****************************
; returns  C if no data available
;         NC if data available
tstc    btst    #rempty,y:<flags
	rts


;****************************
;*        Get a bit         *
;****************************
; returns next bit to be sent in C
; returns Z if this is an end of the transmission
; Note! Interrupts are disabled if end of transmission detected

; check if we are allowed to send data
getbit  jclr    #givedat,y:<flags,xstD2

; check if we must insert a zero
	bclr    #zinsflg,y:<flags
	jcs     getins

; check if there are bits left
	move                y:<xbit,a
	tst     a           y:<xstate,r0
	move                #-1,m0
	jeq     (r0)

; five bit sequence detection logic
getsft0 jclr    #ztstflg,y:<flags,getsft1         ; check if logic enabled
	move                y:<xdata,a
	lsr     a
	move                y:<x5bit,a
	ror     a           #>$f80000,x0
	and     x0,a
	cmp     x0,a        a1,y:<x5bit
	jne     getsft1

; 11111 detected, insert zero
	bset    #zinsflg,y:<flags

; calculate CRC
getsft1 move                y:<xdata,a
	crc     y:<xcrcrem

; shift data out (LSB first) and decrement bit counter
	move                y:<xbit,r0
	move                y:<xdata,a
	lsr     a           (r0)-
	move                r0,y:<xbit
	move                a1,y:<xdata
	andi    #$fb,ccr                            ; NZ
	rts

; insert zero bit
getins  clr     a                                   ; reset five bit counter
	move                a1,y:<x5bit
	andi    #$fa,ccr                            ; NC NZ
	rts


; --- A, after a begin flag
xstA
; set up data xmission
	movi    #xstB,y:<xstate
	movi    #$00ffff,y:<xcrcrem                 ; init CRC generator
	bset    #ztstflg,y:<flags                   ; enable 11111 checker

; --- B, after data byte sent
xstB    movi    #8,y:<xbit                          ; init bit counter for the next byte
	jsr     getchar                             ; fetch next byte
	move    x0,y:<xdata
	move    x0,a
	tst     a
	jpl     getsft0

; last databyte sent, send CRC
	movi    #xstC,y:<xstate
	movi    #16,y:<xbit
	move                y:<xcrcrem,a
	not     a
	move                a1,y:<xdata
	jmp     getsft0

; --- C, after CRC sent
xstC    movi    #xstD,y:<xstate
	movi    #flag,y:<xdata
	movi    #8,y:<xbit
	bclr    #ztstflg,y:<flags                   ; disable 11111 checker
	jmp     getsft0

; --- D, after the last flag sent
xstD    jsr     tstc
	jcs     xstD1

; new data to send, start a new frame
	movi    #xstA,y:<xstate
	movi    #flag,y:<xdata
	movi    #8,y:<xbit
	jmp     getsft0
	rts

; no new data, return with Z (and NC, giving a zero databit)
xstD1   jsr     rdempty                             ; inform xmit control module
xstD2   ori     #$04,ccr
	andi    #$fe,ccr
	rts


;****************************
;*        Put a bit         *
;****************************
; put next bit in C to the host transmit queue
putbit  move                x:<rflag,a
	ror     a           #abrtmsk,b
	move                a1,x:<rflag
	move                a1,x0

; check if abort sequence detected
	and     x0,b        #abort,y0
	eor     y0,b
	jeq     putb4

; check if flag detected
	move                #flagmsk,b
	and     x0,b        #flag,y0
	eor     y0,b        #fivemsk,a
	jeq     putb3                               ; yes, special handling

; check if 11111 sequence detected
	and     x0,a        #five,y0
	eor     y0,a        x0,b
	jeq     putb2                               ; yes, ignore this bit

; no special sequence detected, shift data normally
	jset    #hunt,y:<flags,putb2
	lsl     b           x:<rdata,a
	ror     a           #>@pow(2,-15),x1
	move                a1,x:<rdata
	move                a1,x0

; calculate CRC
	mpy     x0,x1,a     #>1,x1                  ; shift to right 15 bits
	crc     x:<rcrcrem

; decrement the bit counter
	move                x:<rbit,a
	sub     x1,a        #8,b1
	move                a1,x:<rbit
	jne     putb2

; 8 bit shifted, init bit counter again
	move                b1,x:<rbit
	bclr    #firstb,y:<flags
	jcc     putb1

; first byte, init CRC checker
	move                #$00ffff,a1
	move                a1,x:<rcrcrem
	rts

; data bytes, put it to the queue
putb1   bclr    #scndb,y:<flags
	jcs     putb2
	move                x:<rdata,a
	move                #>$ff,x0
	and     x0,a
	move                a1,x0
	jsr     putchar

; discard the previous bit
putb2   rts

; flag detected
putb3   bclr    #hunt,y:<flags
	movi    #8,x:<rbit
	bset    #firstb,y:<flags
	bset    #scndb,y:<flags
	jcc     putb3a                              ; reject frame if it is too short
	jsr     rejc
	rts

; calculate the last CRC bit
putb3a  move                x:<rdata,x0
	move                #>@pow(2,-16),x1        ; shift to right 16 bits
	mpy     x0,x1,a
	crc     x:<rcrcrem

; check that it is valid
	move                #>crcchk,x0
	eor     x0,a
	jne     putb3b                              ; reject frame if CRC failed
	jsr     endc
	rts
putb3b  jsr     rejc
	rts

; abort detected
putb4   bset    #hunt,y:<flags
	jsr     rejc
	rts


;****************************
;*       Request timer      *
;****************************
; delay in x0 (in 1/baud s)
stimer  move                y:<timcnt,a
	add     x0,a        #>$00ffff,x0
	and     x0,a
	move                a1,y:<timval
	bset    #timer,y:<flags
	rts


;****************************
;*    Update output port    *
;****************************
; put lowest eight bits in x0 register to general purpose i/o-port
putio   ori     #$02,mr                             ; disable interrupts
	move                #>$0000ff,a1
	and     x0,a        #>$ffff00,x0
	move    a1,x1

	movep               x:m_pbd,a
	and     x0,a
	or      x1,a
	movep               a1,x:m_pbd

	andi    #$fc,mr
	rts


; *** Persistence routines ***

; state bits
xstart  equ     0                                   ; starting up xmitter (xmit on, no data transmitting)
xstop   equ     1                                   ; putting xmitter off
xon     equ     2                                   ; xmitter currently transmitting data
xoff    equ     3                                   ; xmitter off
xwait   equ     4                                   ; waiting for carrier to be inactive
xpersis equ     5                                   ; waiting for a new persistence algorithm slot

; macro for state setting
nxstate macro   state
	move                #(1<<state),a1
	move                a1,x:<pstate
	endm

; kiss parameters
txdelay equ     1
P       equ     2
SlotTim equ     3
TXtail  equ     4
FullDup equ     5


;****************************
;*   Carrier off routine    *
;****************************
; inform Leonid that there are no data transmissions ongoing
caroff  ori     #$02,mr
	nop
	nop
	nop
	bclr    #carrier,y:<flags

	jsset   #xwait,x:<pstate,dlxmit             ; delayed start of xmitter if we are waiting for it

_car1   andi    #$fc,mr
	rts


;****************************
;*   Carrier on routine     *
;****************************
; inform Leonid that there are data transmissions ongoing
caron   ori     #$02,mr
	nop
	nop
	nop
	bset    #carrier,y:<flags

	jclr    #xpersis,x:<pstate,_car2
	nxstate xwait                               ; fall back to carrier off waiting mode

_car2   andi    #$fc,mr
	rts


; one KISS frame received
frmrec  jset    #xoff,x:<pstate,_frm1
	jset    #xstop,x:<pstate,stxmiti            ; curretly shutting down, but continue directly
	rts

_frm1   move                p:kisspar+FullDup-1,a   ; if fullduplex mode, then there is no need to check the carrier
	tst     a
	jne     stxmit
	jclr    #carrier,y:<flags,dlxmit            ; if no carrier, then start xmitter

	nxstate xwait                               ; carrier, wait until it disappears
	rts

; trying to read empty buffer
rdempty ori     #$02,mr
	nop
	nop
	nop
	jclr    #xon,x:<pstate,_rde1

	nxstate xstop                               ; stop transmitting
	bclr    #givedat,y:<flags
	move                p:kisspar+TXtail-1,a    ; and set txdelay timer
	tst     a           a1,y:<pertim
	jne     _rde1
	jsr     stpxmit                             ; if txtail=0 stop xmitter immediately

_rde1   andi    #$fc,mr
	rts

; persistence algorithm
mkpersi move                a2,x:<a2tmp             ; backup a2 because this routine may be called from interrupt handler
	move                x:(r7),x0               ; first make a random number (combine left audio channel with timer)
	move                #@pow(2,-8),x1
	mpy     x0,x1,a     y:<timcnt,x0
	add     x0,a        #>$0000ff,x0
	and     x0,a
	move    a1,a
	move                p:kisspar+P-1,x0        ; then compare it with P
	cmp     x0,a        x:<a2tmp,a2
	jle     stxmit

	move                p:kisspar+SlotTim-1,a1  ; we lost random number, wait slottime before trying again
	move                a1,y:<pertim
	rts

; wait slottime, then start xmitter
dlxmit  move                p:kisspar+SlotTim-1,a   ; if slottime is zero, start P-persistence algorithm immediately
	tst     a           a1,y:<pertim
	jeq     mkpersi

	nxstate xpersis                             ; else wait first one slottime before starting P-persistence algorithm
	rts

; start xmitter
stxmit  move                y:<xmitsub,r3           ; call user's xmit on routine
	ori     #$01,ccr
	jsr     (r3)

	move                p:kisspar+txdelay-1,a   ; and set txdelay timer
	tst     a           a1,y:<pertim
	jeq     stxmiti                             ; if txdelay=0 then start xmitter immediately
	nxstate xstart                              ; change to XSTART state
	rts

; start xmitter immediately
stxmiti nxstate xon                                 ; start xmitting
	bset    #givedat,y:<flags                   ; start giving data to the application program
	rts

; stop xmitter immediately
stpxmit nxstate xoff                                ; stop xmitting
	move                y:<xmitsub,r3           ; call user's xmit off
	andi    #$fe,ccr
	jsr     (r3)
	rts

; 1/baud s timer ticks here
pertick move                y:<pertim,a1
	move                #>0,x0
	eor     x0,a        a1,r3
	jeq     _pert1
	move                (r3)-                   ; check if timer elapsed
	move    r3,a1
	eor     x0,a        r3,y:<pertim
	jne     _pert1
	jset    #xpersis,x:<pstate,mkpersi
	jset    #xstart,x:<pstate,stxmiti
	jset    #xstop,x:<pstate,stpxmit
_pert1  rts


; read one word (updating CRC)
; returns NC if word received, C if not
rdword  jsr     rdbyte                              ; MS byte
	jcs     _rwe
	move                #>@cvi(@pow(2,8-1)),x1
	mpy     x0,x1,a
	move                a0,y:<tmp
	jsr     rdbyte                              ; LS byte
	jcs     _rwe
	move                y:<tmp,a
	or      x0,a
_rwe    rts


; wait 1s for the next character and update CRC calculation
; returns NC if chr received, C if not
rdbyte  move                #>@cvi(1.0*baud),x0     ; set timer (1s)
	jsr     stimer
	jsr     waitchr
	jcs     _rbe
	crcbyte                                     ; calculate CRC of byte in x0, result in x:<crcrem
	andi    #$fe,ccr                            ; ensure that NC condition is met
_rbe    rts


; wait for (predetermined time) the next character
; returns NC if chr received, C if not
wchr    wait
waitchr jsr     getchar                             ; wait for chr of timer
	jcc     chrfnd
	jset    #timer,y:<flags,wchr
	andi    #$fe,ccr                            ; ensure that NC condition is met
chrfnd  rts                                         ; chr found


; serial buffers
inbuf   dsm     mybuflen
outbuf  dsm     mybuflen


;****************************
;*  CONTEX STORE FOR INTS   *
;****************************

	org     l:$0000

scidata ds      2


	org     x:$0002

rhead   ds      1
rtail   ds      1

tmpblk	dsm	4  ; $0004
a2tmp   ds      1

xhead   ds      1
xtail   ds      1
seqptr  ds      1
crcrem  ds      1

getkst  ds      1
rnhead  ds      1
xnhead  ds      1
kisscmd ds      1

rdata   ds      1                                   ; current byte received
rflag   ds      1                                   ; one bit counter
rbit    ds      1                                   ; received bit counter
rcrcrem ds      1                                   ; CRC remainder

pstate  ds      1                                   ; xmit control module state


	org     y:$0002

flags   ds      1
tmp     ds      1

cryconf ds 4   ; $0004

timcnt  ds      1
timval  ds      1
timchg  ds      1
mspace  ds      1
pgmptr  ds      1

khead   ds      1
kisssub ds      1

xstate  ds      1                                   ; current xmitter state
xdata   ds      1                                   ; current byte to be send
x5bit   ds      1                                   ; one bit counter
xbit    ds      1                                   ; send bits counter
xcrcrem ds      1                                   ; CRC remainder

xmitsub ds      1                                   ; xmit control module xmit routine address store
pertim  ds      1                                   ; xmit control module timer

	endsec  ; bios
	end
