; codec.inc
;-----------------------------------------------------------------------------
;               CODEC CS4215 handling for the DSP56002EVM
;
; V1.0 3.Jan.1997
;-----------------------------------------------------------------------------
;
; EVM CODEC usage definitions - these are only EQU's
NO_PREAMP       equ     $100000
LO_OUT_DRV      equ     $080000
HI_PASS_FILT    equ     $008000
SAMP_RATE_9     equ     $003800         ;  9600    SPS with 24.576MHz xtal
SAMP_RATE_48    equ     $003000         ; 48000    SPS
SAMP_RATE_32    equ     $001800         ; 32000    SPS
SAMP_RATE_27    equ     $001000         ; 27428.57 SPS
SAMP_RATE_16    equ     $000800         ; 16000    SPS
SAMP_RATE_8     equ     $000000         ;  8000    SPS
STEREO          equ     $000400
DATA_8LIN       equ     $200300
DATA_8A         equ     $200200
DATA_8U         equ     $200100
DATA_16         equ     $200000
IMMED_3STATE    equ     $800000
XTAL1_SELECT    equ     $200000  ; ** WRONG **
XTAL2_SELECT    equ     $200000
BITS_64         equ     $000000
BITS_128        equ     $040000
BITS_256        equ     $080000
CODEC_MASTER    equ     $020000
CODEC_TX_OFF    equ     $010000

CTRL_WD_12      equ     NO_PREAMP+HI_PASS_FILT+SAMP_RATE_48+STEREO+DATA_16   ;CLB=0
CTRL_WD_34      equ     IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER
CTRL_WD_56      equ     $000000
CTRL_WD_78      equ     $000000

HEADPHONE_EN    equ     $800000
LINEOUT_EN      equ     $400000
LEFT_ATTN       equ     $010000 ;63*LEFT_ATTN   = -94.5 dB, 1.5 dB steps
SPEAKER_EN      equ     $004000
RIGHT_ATTN      equ     $000100 ;63*RIGHT_ATTN  = -94.5 dB, 1.5 dB steps
MIC_IN_SELECT   equ     $100000
LEFT_GAIN       equ     $010000 ;15*LEFT_GAIN    = 22.5 dB, 1.5 dB steps
MONITOR_ATTN    equ     $001000 ;15*MONITOR_ATTN = mute,    6   dB steps
RIGHT_GAIN      equ     $000100 ;15*RIGHT_GAIN   = 22.5 dB, 1.5 dB steps
OUTPUT_SET      equ     HEADPHONE_EN+LINEOUT_EN+(LEFT_ATTN*4)
INPUT_SET       equ     MIC_IN_SELECT+(15*MONITOR_ATTN)+(RIGHT_ATTN*4)
;-----------------------------------------------------------------------------
; Set codecs input and output settings
;-----------------------------------------------------------------------------
HEADP	equ	$8000
LINEO	equ	$4000
SPEAKER equ	$0040

LINEI	equ	$0000
MIC	equ	$1000
NOHPF   equ     $0000        ; enable input high pass filter
HPF     equ     $8000        ; disable input high pass filter
MICGAIN equ     $100000                        ; Disable 20 dB MIC gain
;MICGAIN equ     $000000                 ; Enable  20 dB MIC gain
;-----------------------------------------------------------------------------
; Open codec
; fs  is the desired sampling rate   (8,9.6,16,27.42857,32,48)
; hpf is the high-pass filter enable (NOHPF,HPF)
;-----------------------------------------------------------------------------
;  Crystal 1 (14,7456MHz):
;    8kHz, 9.6kHz, 16kHz, 27.429kHz, 32kHz, 48kHz
;
;  Crystal 2 (24,576):
;    4.8kHz, 5.76kHz, 9.6kHz, 16.457kHz, 19.2kHz, 28.8kHz, 32.914kHz, 38.4kHz
;
opencd  macro   fs,hpf
  if @def(sim_debug)==0
    if fs==4.8
      move  #>$000000|MICGAIN|hpf,x0
      move  #>$IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==5.76
      move  #>$003800|MICGAIN|hpf,x0
      move  #>$IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==8
      move  #>$000000|MICGAIN|hpf,x0
      move  #>$IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==9.6
      move  #>$003800|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==16.457
      move  #>$001000|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==19.2
      move  #>$001800|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==28.8
      move  #>$003000|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==32
      move  #>$001800|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==32.914
      move  #>$002000|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==38.4
      move  #>$002800|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL1_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    if fs==48.0
      move  #>$003000|MICGAIN|hpf,x0
      move  #>IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER,x1
    endif
    jsr     opencodec
  endif
  endm

;-----------------------------------------------------------------------------
; close codec
;-----------------------------------------------------------------------------
closecd  macro
    jsr closecodec
  endm

;-----------------------------------------------------------------------------
; Set codecs input and output settings
;-----------------------------------------------------------------------------
ctrlcd  macro   init,reg,len,inputs,lgain,rgain,outputs,loattn,roattn
    if @def(sim_debug)==0
      if      (lgain>22.5)|(rgain>22.5)|(loattn>94.5)|(roattn>94.5)
        fail 'Illegal input gain or output attenuation'
      endif
      move                #(inputs|$f0|(@cvi(lgain/1.5)<<8)|@cvi(rgain/1.5))<<8,x0
      clr     a           #(outputs|(@cvi(loattn/1.5)<<8)|@cvi(roattn/1.5))<<8,x1
      move    (reg)+
      do      #len,_initcs
      if      init
        move                a,y:(reg)+
        move                a,y:(reg)+
        else
        move                (reg)+
        move                (reg)+
      endif
      move                x1,y:(reg)+
      move                x0,y:(reg)+
_initcs
      move    (reg)-
    endif
  endm

;-----------------------------------------------------------------------------
; Macro for waiting specified amount of input data from the codec
;-----------------------------------------------------------------------------
waitblk macro	reg,mybuflen,blklen
	if @def(sim_debug)==0
_loop	    ; bset    #0,x:$ffe4
	    wait					; wait for a new sample to be received
	    ; bclr    #0,x:$ffe4
	    move		r7,a
	    move		reg,x0
	    sub     x0,a	#>blklen*4+2,x0
	    jmi     <_wrap
	    cmp     x0,a				; wp - rp > threshold
	    jlo     <_loop
	    jmp     <_ok
_wrap	    move		#>mybuflen*4,x1
	    add     x1,a				; buffer wraparound, rp - wp + lenght > threshold
	    cmp     x0,a
	    jlo     <_loop
_ok
	else
	    movep		x:$ffe0,x:(reg)
	endif
	endm
;-----------------------------------------------------------------------------