// RS232.HPP
// /-------------------------------------------------------------------------\
// |      Klasse zum Ansprechen der RS232-Schnittstelle unter DOS            |
// |                                                                         |
// |                      (C) 30.3.1995 Jrgen Hasch                         |
// \-------------------------------------------------------------------------/
// V1.1a / Borland-C++ 3.1
//

#ifndef __RS232_HPP
#define __RS232_HPP

#ifndef __DOS_H
  #include <DOS.H>
#endif
#ifndef __STDLIB_H
  #include <stdlib.h>
#endif

typedef short WORD;

//
// Gre der internen Puffer
//
#define IBUF_LEN    2048    // Empfangspuffer

//
// Portadressen des 8259 Interrupt-Controllers
//
#define ICR         0x20    // Interrupt Control Register port
#define IMR         0x21    // Interrupt Mask Register port
// Zeichen an 8259 fr 'Ende des Interrupts'
#define EOI         0x20    // End Of Interrupt

//
// Register des 8250 oder 16550
//
//   TXR  Output data to the serial port
//   RXR  Input data from the serial port
//   FCR  Fifo control register
//   LCR  Initialize the serial port
//   IER  Controls interrupt generation
//   IIR  Identifies interrupts
//   MCR  Send control signals to the modem
//   LSR  Monitor the status of the serial port
//   MSR  Receive status of the modem
#define TXR         0x00    // Transmit Register    (WRITE)
#define RXR         0x00    // Receive Register     (READ)
#define IER         0x01    // Interrupt Enable Register
#define IIR         0x02    // Interrupt ID Register
#define FCR         0x02    // Fifo control register
#define LCR         0x03    // Line Control Register
#define MCR         0x04    // Modem Control Register
#define LSR         0x05    // Line Status Register
#define MSR         0x06    // Modem Status Register

//
// Bitkonstanten
//
#define DTR         0x01    // Data Terminal Ready
#define RTS         0x02    // Request To Send.  There is data to send.
#define MCI         0x08    // Modem Control Interrupt

#define CTS         0x10    // Clear To Send
#define DSR         0x20    // Data Set Ready

static unsigned char COM_IRQ[] = { 0,
				4,     // COM1
				3,     // COM2
				4,     // COM3
				3      // COM4
			      };
static unsigned int COM_BASE[] = { 0,
				    0x3f8, // COM1
				    0x2f8, // COM2
				    0x3e8, // COM3
				    0x2e8  // COM4
				  };

//
// Klassendefinition fr RS232-Klasse
//
class RS232 {
private:
    //
    // private Variablen
    //
    int             irq;                // HW-Interrupt
    unsigned long   baud;               // maximum baud speed for modem
    int             fifoen;             // 1 wenn FIFO vorhanden
    //
    // private Memberfunktionen
    //
    void RS232Init();                   // Schnittstelle initialisieren
    void dotx(char);                    // private Sendefunktion
    char dorx();                        // private Empfangsfunktion
    void my_setvect(unsigned char numm, void far interrupt (RS232::*func)(...));
    void far interrupt (*OldVect)(...); // old int handler
    void far interrupt fifo_irq(...);
    void far interrupt nofifo_irq(...);
public:
    //
    // Liste der seriellen Schnittstellen
    //
    enum COM_PORTS {
	COM1 =  0x01,   // Com port 1
	COM2 =  0x02,   // Com port 2
	COM3 =  0x03,   // Com port 3
	COM4 =  0x04    // Com port 4
    };
    //
    // Konstruktoren
    //
    RS232(unsigned char comport ,unsigned long baudrate=19200L);
    RS232(unsigned long baudrate, int portaddr, int portint);
    //
    // Destruktor
    //
    ~RS232();
    //
    // ffentliche Memberfunktionen
    //
    void setBaud(unsigned long);     // Baudrate einstellen
    int dtr(void);                  // DTR-Status abfragen
    inline void setDtr(void);       // DTR setzen
    void dropDtr(void);             // DTR lschen

    //
    // 'C'-Funktionen
    //
    inline char rxchar(void);              // Zeichen empfangen
    inline WORD rxword(void);              // Wort empfangen
    inline void txchar(char ch);            // Zeichen senden
    inline void txword(WORD wrd);          // Wort senden

    void txstring(char *);                 // String senden
    void printf(char *, ...);              // formatierten String senden
    //
    // --- Inline-Funktionen ---
    //
    // liefert Anzahl der Zeichen im Empfangspuffer
    //
    int  inCount(void);
    //
    // Empfangspuffer leeren
    //
    void flushInBuf(void);
    //
    // liefert 1, wenn FIFO vorhanden
    //
    int isfifo(void)
      { return fifoen; }
};

#endif // __RS232_HPP
